import pytest


@pytest.fixture(scope="session")
def sample_dfxp():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
  <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="bottom">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    \u266a ...say bow, wow, \u266a
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="bottom" style="p">
    <span tts:textalign="right">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="bottom" style="p">
   <br/>
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="bottom" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="bottom" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:36.200" region="bottom" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_inline_style():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
  <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="bottom">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    This is <span tts:fontstyle="italic">italic</span>, <span tts:fontweight="bold">bold
    </span>, <span tts:textdecoration="overline underline">
    underline</span>, <span tts:fontstyle="italic" tts:fontweight="bold" tts:textdecoration="underline">
    everything together in one tag</span>, and <span tts:textdecoration="underline">
    <span tts:fontweight="bold"><span tts:fontstyle="italic">nested</span></span></span>.
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_with_defined_style():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
   <style xml:id="s1" tts:fontstyle="italic"/>
   <style xml:id="s2" tts:fontweight="bold" />
   <style xml:id="s3" tts:textdecoration="underline" />
  </styling>
  <layout>
  <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="bottom">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    This is <span style="s1">italic</span>, <span style="s2">bold</span>, <span style="s3">
    underline</span>, <span style="s1 s2 s3">everything together in one tag</span>, and <span style="s3">
    <span style="s2"><span style="s1">nested</span></span></span>.
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_with_inherited_style():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
   <style xml:id="s1" tts:fontstyle="italic"/>
   <style xml:id="s2" tts:fontweight="bold" />
   <style xml:id="s3" tts:textdecoration="underline" />
   <style xml:id="inherit_s3" style="s3" />
   <style xml:id="inherit_s3_underline" style="s1" tts:textdecoration="underline" />
   <style xml:id="inherit_s3_bold" style="inherit_s3_underline" tts:fontweight="bold" />
   <style xml:id="inherit_s2" style="s2" />
   <style xml:id="inherit_s1" style="s1" />
  </styling>
  <layout>
  <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="bottom">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    This is <span style="inherit_s1">italic</span>, <span style="inherit_s2">bold</span>, <span style="inherit_s3">
    underline</span>, <span style="inherit_s3_bold">everything together in one tag</span>, and <span style="inherit_s3">
    <span style="inherit_s2"><span style="inherit_s1">nested</span></span></span>.
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_without_region_and_style():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
 </head>
 <body>
  <div xml:lang="en-US">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="bottom" style="p">
    <span tts:textalign="right">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="bottom" style="p">
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="bottom" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="bottom" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:34.400" region="bottom" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
   <p begin="00:00:34.400" end="00:00:38.400" region="bottom" style="p">some more text</p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_positioning():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en-US"
    xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts='http://www.w3.org/ns/ttml#styling'
    >
<head>
    <styling>
        <style xml:id="padded" tts:backgroundColor="yellow" tts:padding="10%"/>
    </styling>
    <layout>
        <region xml:id="regionOne" tts:textAlign='middle' tts:origin='96px 54px' tts:extent='70% 25%' style="padded"/>
        <region xml:id="regionTwo" tts:backgroundColor="blue" tts:textAlign='right' tts:origin='25% 25%' tts:extent='50% 10%'/>
        <region xml:id="regionThree" tts:backgroundColor="green" tts:textAlign='left' tts:origin='50% 50%' tts:extent='25% 25%'/>
        <region xml:id="regionFour" tts:backgroundColor="red" tts:textAlign='right' tts:origin='25% 75%' tts:extent='25% 25%'/>
    </layout>
</head>
<body>
    <div>
        <p region="regionOne" tts:backgroundColor="black" begin='00:00:01.000' end='00:00:03.000'>
        You might not remember us. We are a typical transparent region with centered text that has an outline.
        </p>
        <p region="regionTwo" begin='00:00:03.500' end='00:00:05.000'>
        had <span tts:textDecoration="underline">personality.</span>
        </p>
        <p region="regionThree" begin='00:00:05.500' end='00:00:07.000'>
        Hello there, children! Have you seen any visitors?
        </p>
        <p region="regionFour" begin='00:00:07.500' end='00:00:09.000'>
        This is<br/>
        the last cue
        </p>
    </div>
</body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_relativized_positioning():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling/>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:extent="70% 25%" tts:origin="15% 15%" tts:padding="10% 10% 10% 10%" xml:id="r0"/>
   <region tts:displayAlign="after" tts:extent="50% 10%" tts:origin="25% 25%" tts:textAlign="right" xml:id="r1"/>
   <region tts:displayAlign="after" tts:extent="25% 25%" tts:origin="50% 50%" tts:textAlign="left" xml:id="r2"/>
   <region tts:displayAlign="after" tts:extent="25% 20%" tts:origin="25% 75%" tts:textAlign="right" xml:id="r3"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:01.000" end="00:00:03.000" region="r0">
    You might not remember us. We are a typical transparent region with centered text that has an outline.
   </p>
   <p begin="00:00:03.500" end="00:00:05.000" region="r1">
    had <span region="r1">personality.</span>
   </p>
   <p begin="00:00:05.500" end="00:00:07.000" region="r2">
    Hello there, children! Have you seen any visitors?
   </p>
   <p begin="00:00:07.500" end="00:00:09.000" region="r3">
    This is<br/>
    the last cue
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_empty():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontfamily="Arial"
          tts:fontsize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US">
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_syntax_error():
    return """
<?xml version="1.0" encoding="UTF-16"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml">
<body>
  <div>
    <p begin="0:00:02.07" end="0:00:05.07">>>THE GENERAL ASSEMBLY'S 2014</p>
    <p begin="0:00:05.07" end="0:00:06.21">SESSION GOT OFF TO A LATE START,</p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_from_sami_with_positioning():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
   <region tts:padding="2pt 1pt 2pt 1pt" tts:textalign="center" xml:id="r0"></region>
   <region tts:padding="2pt 1pt 2pt 1pt" tts:textalign="right" xml:id="r1"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="r0">
   <p begin="00:00:09.209" end="00:00:12.312" region="r0" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="r0" style="p">
    MAN:<br/>
    When we think<br/>
    \u266a ...say bow, wow, \u266a
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="r1" style="p">
    we have this vision of Einstein
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="r0" style="p">
   <br/>
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="r0" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="r0" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:36.200" region="r0" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_long_cue():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:backgroundColor="black" tts:color="white" tts:fontFamily="monospace" tts:fontSize="8%" xml:id="basic"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:origin="25% 25%" tts:textAlign="center" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div>
   <p begin="00:00:01.000" end="00:00:02.000" region="bottom" style="basic">
    NARRATOR:
   </p>
   <p begin="00:00:02.000" end="00:00:03.000" region="r0" style="basic">
    They built the largest, most incredible, wildest, craziest,
   </p>
   <p begin="00:00:03.000" end="00:00:04.000" region="bottom" style="basic">
    most complex machine in history.
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_long_cue_fit_to_screen():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="white" tts:fontFamily="monospace" tts:fontSize="8%" xml:id="basic"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:extent="65% 70%" tts:origin="25% 25%" tts:textAlign="center" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en">
   <p begin="00:00:01.000" end="00:00:02.000" region="bottom" style="basic">
    NARRATOR:
   </p>
   <p begin="00:00:02.000" end="00:00:03.000" region="r0" style="basic">
    They built the largest, most incredible, wildest, craziest,
   </p>
   <p begin="00:00:03.000" end="00:00:04.000" region="bottom" style="basic">
    most complex machine in history.
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_from_sami_with_margins():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffffff" tts:fontFamily="Tahoma" tts:fontSize="24pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:padding="0% 29pt 0% 29pt" tts:textAlign="center" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="r0" xml:lang="en-US">
   <p begin="00:00:00.133" end="00:00:04.133" region="r0" style="p">
    &gt;&gt; COMING UP NEXT, IT IS<br/>
    APPLAUSE AMERICA.
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_from_sami_with_lang_margins():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffffff" tts:fontFamily="Tahoma" tts:fontSize="24pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:padding="20px 20px 20px 20px" tts:textAlign="center" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="r0" xml:lang="en-US">
   <p begin="00:00:00.133" end="00:00:04.133" region="r0" style="p">
    &gt;&gt; COMING UP NEXT, IT IS<br/>
    APPLAUSE AMERICA.
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_from_sami_with_span():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffffff" tts:fontFamily="Tahoma" tts:fontSize="24pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:padding="20px 20px 20px 20px" tts:textAlign="center" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="r0" xml:lang="en-US">
   <p begin="00:00:00.133" end="00:00:04.133" region="r0" style="p">
    <span tts:fontSize="36pt">we have this vision of Einstein</span>
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_from_sami_with_bad_span_align():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffffff" tts:fontFamily="Tahoma" tts:fontSize="24pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:padding="20px 20px 20px 20px" tts:textAlign="center" xml:id="r0"/>
   <region tts:padding="20px 20px 20px 20px" tts:textAlign="right" xml:id="r1"/>
  </layout>
 </head>
 <body>
  <div region="r0" xml:lang="en-US">
   <p begin="00:00:00.133" end="00:00:04.133" region="r1" style="p">
    Some say we have this vision of Einstein as an old, wrinkly man
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def dfxp_style_region_align_conflict():
    return """<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffffff" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:textAlign="left" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="r0" xml:lang="en-US">
   <p begin="00:00:04.537" end="00:00:07.841" region="r0" style="p">
    IT'S WORD GIRL♫
   </p>
   <p begin="00:00:08.537" end="00:00:10.841" region="r0" style="p">
    ♫WORD UP,<br/>
    IT'S WORD GIRL♫
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_invalid_but_supported_positioning_input():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontFamily="Arial"
          tts:fontSize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US">
   <p tts:origin="17.5% 10%" tts:extent="62.5% 5.33%" begin="0:00:02.777" end="0:00:05.700">
   Hello there!
   </p>
   <p tts:origin="20% 15.67%" tts:extent="30% 7.67%" begin="0:00:05.700" end="0:00:06.210">
   How are you?<span tts:origin="1px 2px">>>Fine, thx<<</span>
   </p>
   <p tts:extent="60% 22%" begin="0:00:07.900" end="0:00:08.900" tts:textAlign="right" tts:displayAlign="before">
   Just fine?
   </p>
   <p tts:origin="11% 11%" begin="0:00:09.900" end="0:00:10.800">
    <span weirdAttribute="1" tts:textAlign="right">>>>Lol, yes!</span>
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_invalid_but_supported_positioning_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:extent="62.5% 5.33%" tts:origin="17.5% 10%" tts:textAlign="center" xml:id="r0"/>
   <region tts:displayAlign="after" tts:extent="30% 7.67%" tts:origin="20% 15.67%" tts:textAlign="center" xml:id="r1"/>
   <region tts:displayAlign="after" tts:extent="30% 7.67%" tts:origin="1px 2px" tts:textAlign="center" xml:id="r2"/>
   <region tts:displayAlign="before" tts:extent="60% 22%" tts:textAlign="right" xml:id="r3"/>
   <region tts:displayAlign="after" tts:origin="11% 11%" tts:textAlign="center" xml:id="r4"/>
   <region tts:displayAlign="after" tts:origin="11% 11%" tts:textAlign="right" xml:id="r5"/>
  </layout>
 </head>
 <body>
  <div region="bottom" tts:displayAlign="after" tts:textAlign="center" xml:lang="en-US">
   <p begin="00:00:02.777" end="00:00:05.700" region="r0" style="p" tts:displayAlign="after" tts:extent="62.5% 5.33%" tts:origin="17.5% 10%" tts:textAlign="center">
    Hello there!
   </p>
   <p begin="00:00:05.700" end="00:00:06.210" region="r1" style="p" tts:displayAlign="after" tts:extent="30% 7.67%" tts:origin="20% 15.67%" tts:textAlign="center">
    How are you?<span region="r2" tts:origin="1px 2px" tts:textAlign="center" tts:extent="30% 7.67%" tts:displayAlign="after">&gt;&gt;Fine, thx&lt;&lt;</span>
   </p>
   <p begin="00:00:07.900" end="00:00:08.900" region="r3" style="p" tts:displayAlign="before" tts:extent="60% 22%" tts:textAlign="right">
    Just fine?
   </p>
   <p begin="00:00:09.900" end="00:00:10.800" region="r4" style="p" tts:displayAlign="after" tts:origin="11% 11%" tts:textAlign="center">
    <span tts:textAlign="right" region="r5" tts:origin="11% 11%" tts:textAlign="right" tts:displayAlign="after">&gt;&gt;&gt;Lol, yes!</span>
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_multiple_regions_input():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontfamily="Arial"
          tts:fontsize="10pt" tts:textAlign="center"/>
    <style xml:id="referential_style2" tts:extent="3em 4em"/>
    <style xml:id="referential_style1" tts:padding="3px 4px 5px" style="referential_style2"/>
  </styling>
  <layout>
   <region tts:textAlign="center" xml:id="pixel_region">
    <style tts:origin="40px 50px" tts:extent="30px 40px"/>
   </region>
   <region tts:origin="10% 30%" tts:extent="50% 50%" xml:id="percent_region"/>
   <region tts:padding="2c" xml:id="padding_region" />
   <region xml:id="referential_region" style="referential_style1"/>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US">
   <p region="pixel_region" begin="0:00:02.700" end="0:00:05.700">
   Hello there!
   </p>
   <p region="percent_region" begin="0:00:05.700" end="0:00:06.210">
   How are you?
   </p>
   <p region="padding_region" begin="0:00:07.700" end="0:00:09.210">
   >> I'm fine, thank you << replied someone.
   <span region="percent_region">
    >>And now we're going to have fun<<
    </span>
   </p>
   <p region="referential_region" begin="0:00:10.707" end="0:00:11.210">
   What do you have in mind?
   </p>
   <p begin="0:00:12.900" end="0:00:13.900" tts:textAlign="start"
        tts:displayAlign="after">
   To write random words here!
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_multiple_regions_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:extent="30px 40px" tts:origin="40px 50px" tts:textAlign="center" xml:id="r0"/>
   <region tts:displayAlign="after" tts:extent="50% 50%" tts:origin="10% 30%" tts:textAlign="center" xml:id="r1"/>
   <region tts:displayAlign="after" tts:padding="2c 2c 2c 2c" tts:textAlign="center" xml:id="r2"/>
   <region tts:displayAlign="after" tts:extent="3em 4em" tts:padding="3px 4px 5px 4px" tts:textAlign="center" xml:id="r3"/>
   <region tts:displayAlign="after" tts:textAlign="start" xml:id="r4"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:02.700" end="00:00:05.700" region="r0" style="p">
    Hello there!
   </p>
   <p begin="00:00:05.700" end="00:00:06.210" region="r1" style="p">
    How are you?
   </p>
   <p begin="00:00:07.700" end="00:00:09.210" region="r2" style="p">
    &gt;&gt; I'm fine, thank you &lt;&lt; replied someone.<span region="r1">&gt;&gt;And now we're going to have fun&lt;&lt;</span>
   </p>
   <p begin="00:00:10.707" end="00:00:11.210" region="r3" style="p">
    What do you have in mind?
   </p>
   <p begin="00:00:12.900" end="00:00:13.900" region="r4" style="p" tts:textAlign="start">
    To write random words here!
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_to_render_with_only_default_positioning_input():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p" tts:origin="10% 20%" tts:extent="10% 30%"/>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="left" xml:id="p2"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:extent="30px 40px" tts:origin="40px 50px" tts:textAlign="center" xml:id="r0"/>
   <region tts:displayAlign="after" tts:extent="50% 50%" tts:origin="10% 30%" tts:textAlign="center" xml:id="r1"/>
   <region tts:displayAlign="after" tts:padding="2c 2c 2c 2c" tts:textAlign="center" xml:id="r2"/>
   <region tts:displayAlign="after" tts:extent="3em 4em" tts:padding="3px 4px 5px 4px" tts:textAlign="center" xml:id="r3"/>
   <region tts:displayAlign="after" tts:textAlign="start" xml:id="r4"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:02.700" end="00:00:05.700" region="r0" style="p">
    Hello there!
   </p>
   <p begin="00:00:05.700" end="00:00:06.210" region="r1" style="p2">
    How are you?
   </p>
  </div>
 </body>
</tt>"""

##
# When converting from DFXP to DFXP, notice the extra region "r0" is added, to
# support the spam that sets the "tts:textAlign" attribute.
# This attribute is meaningless on the <span> tag, and should affect only <p>
# tags. It's a feature that should affect only only those players that don't
# conform to the specs.


# UNUSED SAMPLE
@pytest.fixture(scope="session")
def sample_dfxp_to_dfxp_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:textAlign="right" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="bottom" style="p">
    <span tts:textAlign="right" region="r0">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="bottom" style="p">
    <br/>
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="bottom" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="bottom" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:34.400" region="bottom" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
   <p begin="00:00:34.400" end="00:00:38.400" region="bottom" style="p">
    some more text
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:textAlign="right" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    \u266a ...say bow, wow, \u266a
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="bottom" style="p">
    <span tts:textAlign="right" region="r0">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="bottom" style="p">
    <br/>
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="bottom" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="bottom" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:36.200" region="bottom" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_style_tag_with_no_xml_id_input():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" xml:id="r0">
    <style tts:textAlign="start"/>
   </region>
  </layout>
 </head>
 <body>
  <div xml:lang="en">
   <p begin="00:00:09.209" end="00:00:12.312" region="r0" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_style_tag_with_no_xml_id_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="after" tts:textAlign="start" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en">
   <p begin="00:00:09.209" end="00:00:12.312" region="r0" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_from_scc_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="white" tts:fontFamily="monospace" tts:fontSize="1c" xml:id="default"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="before" tts:origin="10% 77%" tts:textAlign="left" xml:id="r0"/>
   <region tts:displayAlign="before" tts:origin="40% 5%" tts:textAlign="left" xml:id="r1"/>
   <region tts:displayAlign="before" tts:origin="70% 23%" tts:textAlign="left" xml:id="r2"/>
   <region tts:displayAlign="before" tts:origin="20% 47%" tts:textAlign="left" xml:id="r3"/>
   <region tts:displayAlign="before" tts:origin="20% 89%" tts:textAlign="left" xml:id="r4"/>
   <region tts:displayAlign="before" tts:origin="40% 53%" tts:textAlign="left" xml:id="r5"/>
   <region tts:displayAlign="before" tts:origin="70% 17%" tts:textAlign="left" xml:id="r6"/>
   <region tts:displayAlign="before" tts:origin="20% 35%" tts:textAlign="left" xml:id="r7"/>
   <region tts:displayAlign="before" tts:origin="20% 83%" tts:textAlign="left" xml:id="r8"/>
   <region tts:displayAlign="before" tts:origin="70% 11%" tts:textAlign="left" xml:id="r9"/>
   <region tts:displayAlign="before" tts:origin="40% 41%" tts:textAlign="left" xml:id="r10"/>
   <region tts:displayAlign="before" tts:origin="20% 71%" tts:textAlign="left" xml:id="r11"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:01.167" end="00:00:03.103" region="r0" style="default">
    abab
   </p>
   <p begin="00:00:01.167" end="00:00:03.103" region="r1" style="default">
    cdcd
   </p>
   <p begin="00:00:01.167" end="00:00:03.103" region="r2" style="default">
    efef
   </p>
   <p begin="00:00:03.169" end="00:00:09.743" region="r3" style="default">
    ghgh<br/>
    ijij<br/>
    klkl
   </p>
   <p begin="00:00:09.743" end="00:00:11.745" region="r4" style="default">
    mnmn
   </p>
   <p begin="00:00:09.743" end="00:00:11.745" region="r5" style="default">
    opop
   </p>
   <p begin="00:00:09.743" end="00:00:11.745" region="r6" style="default">
    qrqr
   </p>
   <p begin="00:00:11.745" end="00:00:20.120" region="r7" style="default">
    stst<br/>
    uvuv<br/>
    wxwx
   </p>
   <p begin="00:00:20.120" end="00:00:22.122" region="r8" style="default">
    yzyz
   </p>
   <p begin="00:00:20.120" end="00:00:22.122" region="r9" style="default">
    0101
   </p>
   <p begin="00:00:20.120" end="00:00:22.122" region="r10" style="default">
    2323
   </p>
   <p begin="00:00:22.122" end="00:00:26.122" region="r11" style="default">
    4545<br/>
    6767<br/>
    8989
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_properly_closing_spans_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="white" tts:fontFamily="monospace" tts:fontSize="1c" xml:id="default"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="before" tts:extent="70% 12%" tts:origin="20% 83%" tts:textAlign="left" xml:id="r0"/>
   <region tts:displayAlign="before" tts:extent="60% 12%" tts:origin="30% 83%" tts:textAlign="left" xml:id="r1"/>
   <region tts:displayAlign="before" tts:extent="40% 12%" tts:origin="50% 83%" tts:textAlign="left" xml:id="r2"/>
   <region tts:displayAlign="before" tts:extent="30% 12%" tts:origin="60% 83%" tts:textAlign="left" xml:id="r3"/>
   <region tts:displayAlign="before" tts:extent="60% 6%" tts:origin="30% 89%" tts:textAlign="left" xml:id="r4"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:01:31.400" end="00:01:35.666" region="r0" style="default">
    cccccc<br/>
    c!c!
   </p>
   <p begin="00:01:35.666" end="00:01:40.866" region="r1" style="default">
    bbbb
   </p>
   <p begin="00:01:35.666" end="00:01:40.866" region="r2" style="default">
    <span tts:fontStyle="italic" region="r2">cccc<br/>
    bbaa</span>
   </p>
   <p begin="00:01:55.800" end="00:01:59.533" region="r0" style="default">
    aa
   </p>
   <p begin="00:01:55.800" end="00:01:59.533" region="r3" style="default">
    <span tts:fontStyle="italic" region="r3">bb<br/>
    cc</span>
   </p>
   <p begin="00:01:59.533" end="00:02:03.533" region="r3" style="default">
    abcd
   </p>
   <p begin="00:01:59.533" end="00:02:03.533" region="r4" style="default">
    abcd
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_for_legacy_writer_input():
    return """
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="p" tts:color="#ffeedd" tts:fontfamily="Arial"
          tts:fontsize="10pt" tts:textAlign="center"/>
  </styling>
  <layout>
   <region xml:id="r0" tts:origin="20% 30%" tts:extent="30% 30%"/>
  </layout>
 </head>
 <body>
  <div xml:lang="en">
   <p begin="00:00:09.209" end="00:00:12.312" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="r0" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" style="p">
    <span tts:textAlign="right">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" style="p">
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" tts:origin="10% 10%" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" tts:testAlign="left" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:34.400" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
   <p begin="00:00:34.400" end="00:00:38.400" style="p">some more text</p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_for_legacy_writer_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="p"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
  </layout>
 </head>
 <body>
  <div xml:lang="en">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom" style="p">
    ( clock ticking )
   </p>
   <p begin="00:00:14.848" end="00:00:17.000" region="bottom" style="p">
    MAN:<br/>
    When we think<br/>
    of "E equals m c-squared",
   </p>
   <p begin="00:00:17.000" end="00:00:18.752" region="bottom" style="p">
    <span tts:textAlign="right">we have this vision of Einstein</span>
   </p>
   <p begin="00:00:18.752" end="00:00:20.887" region="bottom" style="p">
    as an old, wrinkly man<br/>
    with white hair.
   </p>
   <p begin="00:00:20.887" end="00:00:26.760" region="bottom" style="p">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
   </p>
   <p begin="00:00:26.760" end="00:00:32.200" region="bottom" style="p">
    MAN 2:<br/>
    It's all about an eternal Einstein.
   </p>
   <p begin="00:00:32.200" end="00:00:34.400" region="bottom" style="p">
    &lt;LAUGHING &amp; WHOOPS!&gt;
   </p>
   <p begin="00:00:34.400" end="00:00:38.400" region="bottom" style="p">
    some more text
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def dfxp_with_concurrent_captions():
    return """\
<tt xml:lang="en-US"
    xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts='http://www.w3.org/ns/ttml#styling'
    >
<head>
    <layout>
        <region xml:id="r0" tts:textAlign="center" tts:displayAlign="after" tts:origin="5% 5%" tts:extent="90% 90%"/>
        <region xml:id="r1" tts:textAlign="center" tts:displayAlign="after" tts:origin="5% 15%" tts:extent="90% 80%"/>
    </layout>
</head>
<body>
    <div>
        <p region="r0" begin='00:00:01.000' end='00:00:03.000'>
        When we think
        </p>
        <p region="r1" begin='00:00:01.000' end='00:00:03.000'>
        of "E equals m c-squared",
        </p>
        <p region="r0" begin='00:00:03.000' end='00:00:04.000'>
        we have this vision of Einstein
        </p>
        <p region="r0" begin='00:00:04.000' end='00:00:05.000'>
        as an old, wrinkly man
        </p>
        <p region="r1" begin='00:00:04.000' end='00:00:05.000'>
        with white hair.
        </p>
    </div>
</body>
</tt>"""


# 'style_name' is the template parameter to use in str.format
@pytest.fixture(scope="session")
def sample_dfxp_with_templated_style():
    return """\
<tt xml:lang="en-US"
    xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts='http://www.w3.org/ns/ttml#styling'
    >
<head>
    <layout>
        <region xml:id="r0" tts:textAlign="center" tts:displayAlign="after" tts:origin="5% 5%" tts:extent="90% 90%"/>
    </layout>
    <styling>
        <style tts:color="#ffeedd" tts:fontFamily="Arial" tts:fontSize="10pt" tts:textAlign="center" xml:id="{style_name}"/>
    </styling>
</head>
<body>
    <div>
        <p region="r0" begin="00:00:01.000" end="00:00:03.000" style="{style_name}">
        When we think
        </p>
    </div>
</body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_with_escaped_apostrophe():
    return """\
<tt xml:lang="en-US"
    xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts='http://www.w3.org/ns/ttml#styling'
    >
<head>
    <layout>
        <region xml:id="r0" tts:textAlign="center" tts:displayAlign="after" tts:origin="5% 5%" tts:extent="90% 90%"/>
    </layout>
</head>
<body>
    <div>
        <p region="r0" begin='00:00:01.000' end='00:00:03.000'>
        << &quot;Andy&apos;s Caf&eacute; & Restaurant&quot; this way
        </p>
    </div>
</body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_alternative_timing_formats():
    return """\
<tt xml:lang="en-US"
    xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts='http://www.w3.org/ns/ttml#styling'
    >
<head>
    <layout>
        <region xml:id="r0" tts:textAlign="center" tts:displayAlign="after" tts:origin="5% 5%" tts:extent="90% 90%"/>
    </layout>
</head>
<body>
    <div>
        <p region="r0" begin='00:00:01.9' end='00:00:03.05'>
            foo
        </p>
        <p region="r0" begin='4s' end='5.2s'>
            bar
        </p>
    </div>
</body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_empty_paragraph():
    return """
<?xml version="1.0" encoding="UTF-16"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml">
<body>
  <div>
    <p begin="0:00:02.07" end="0:00:05.07"></p>
    <p begin="0:00:05.07" end="0:00:06.21">SESSION GOT OFF TO A LATE START,</p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_incorrect_time_format():
    return """
<?xml version="1.0" encoding="UTF-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml">
<body>
  <div>
    <p begin="0:00:02.07" end="0:05">Hey! Check out this error!</p>
  </div>
 </body>
</tt>
"""


@pytest.fixture(scope="session")
def sample_dfxp_with_frame_timing():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <layout>
   <region tts:origin="10% 10%" xml:id="b1"/>
   <region tts:origin="40% 40%" xml:id="b2"/>
   <region tts:origin="10% 70%" xml:id="b3"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:09:20" end="00:00:12:7" region="b1">
    ABC
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_empty_cue():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <layout>
   <region tts:origin="10% 10%" xml:id="bottom"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:01.209" end="00:00:02.312" region="bottom">abc</p>
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom"></p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_empty_cue_output():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
   <styling>
      <style tts:color="white" tts:fontFamily="monospace" tts:fontSize="1c" xml:id="default"/>
   </styling>
   <layout>
     <region tts:displayAlign="after" tts:origin="10% 10%" tts:textAlign="center" xml:id="r0"/>
     <region tts:displayAlign="after" tts:extent="80% 85%" tts:origin="10% 10%" tts:textAlign="center" xml:id="r1"/>
   </layout>
 </head>
 <body>
   <div region="r0" xml:lang="en-US">
     <p begin="00:00:01.209" end="00:00:02.312" region="r1" style="default">
       abc
     </p>
   </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_invalid_positioning_value_template():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <layout>
   <region tts:origin="{origin}" xml:id="bottom"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:09.209" end="00:00:12.312" region="bottom">
    ( clock ticking )
   </p>
  </div>
 </body>
</tt>"""


# TODO - notice that there's no "bottom" region specified in the <layout>
# region, but it's referenced by the <div>. Decide if this is ok enough
@pytest.fixture(scope="session")
def sample_dfxp_multiple_captions_with_the_same_timing():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <layout>
   <region tts:origin="10% 10%" xml:id="b1"/>
   <region tts:origin="40% 40%" xml:id="b2"/>
   <region tts:origin="10% 70%" xml:id="b3"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:00:09.209" end="00:00:12.312" region="b1">
    Some text here
   </p>
   <p begin="00:00:09.209" end="00:00:12.312" region="b2">
    Some text there
   </p>
   <p begin="00:00:09.209" end="00:00:12.312" region="b3">
    Caption texts are everywhere!
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_ampersand_character():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml" xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style tts:color="white" tts:fontFamily="monospace" tts:fontSize="1c" xml:id="default"/>
  </styling>
  <layout>
   <region tts:displayAlign="after" tts:textAlign="center" xml:id="bottom"/>
   <region tts:displayAlign="before" tts:extent="70% 90%" tts:origin="20% 5%" tts:textAlign="left" xml:id="r0"/>
  </layout>
 </head>
 <body>
  <div region="bottom" xml:lang="en-US">
   <p begin="00:04:36.833" end="00:04:40.833" region="r0" style="default">
    MAJOR FUNDING PROVIDED BY &amp;
   </p>
  </div>
 </body>
</tt>"""


@pytest.fixture(scope="session")
def sample_dfxp_with_nested_spans():
    return """\
<?xml version="1.0" encoding="utf-8"?>
<tt xml:lang="en" xmlns="http://www.w3.org/ns/ttml"
    xmlns:tts="http://www.w3.org/ns/ttml#styling">
 <head>
  <styling>
   <style xml:id="s1" tts:fontstyle="italic"/>
   <style xml:id="s2" tts:fontweight="bold" />
   <style xml:id="s3" tts:textdecoration="underline" />
  </styling>
  <layout>
  <region xml:id="bottom" tts:displayAlign="after" tts:textAlign="center"></region>
  </layout>
 </head>
 <body>
  <div xml:lang="en-US" region="bottom">
   <p begin="00:00:03.209" end="00:00:8.312" region="bottom">
    That is <span style="s3"><span style="s2"><span style="s1">nested</span></span></span>.
   </p>
  </div>
 </body>
</tt>"""



import pytest


@pytest.fixture(scope="session")
def sample_sami():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE><STYLE TYPE="text/css">
<!--
P { margin-left:  1pt;
    margin-right: 1pt;
    margin-bottom: 2pt;
    margin-top: 2pt;
    text-align: center;
    font-size: 10pt;
    font-family: Arial;
    font-weight: normal;
    font-style: normal;
    color: #ffeedd; }

.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}

--></STYLE></HEAD><BODY>
<SYNC start="9209"><P class="ENCC">
       ( clock ticking )
</P></SYNC>
<SYNC start="12312"><P class="ENCC">&nbsp;</P></SYNC>
<SYNC start="14848"><P class="ENCC">
    MAN:<br/>
    When we think<br/>
    \u266a ...say bow, wow, \u266a
</P></SYNC>
<SYNC start="17000"><P class="ENCC">
  <SPAN Style="text-align:right;">we have this vision of Einstein</SPAN>
</P></SYNC>
<SYNC start="18752"><P class="ENCC">
    <br/>
    as an old, wrinkly man<br/>
    with white hair.
</P></SYNC>
<SYNC start="20887"><P class="ENCC">
    MAN 2:<br/>
    E equals m c-squared is<br/>
    not about an old Einstein.
</P></SYNC>
<SYNC start="26760"><P class="ENCC">
    MAN 2:<br/>
    It's all about an eternal Einstein.
</P></SYNC>
<SYNC start="32200"><P class="ENCC">
    &lt;LAUGHING &amp; WHOOPS!&gt;
</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_style_tags():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE><STYLE TYPE="text/css">
<!--
P { margin-left:  1pt;
    margin-right: 1pt;
    margin-bottom: 2pt;
    margin-top: 2pt;
    text-align: center;
    font-size: 10pt;
    font-family: Arial;
    font-weight: normal;
    font-style: normal;
    color: #ffeedd; }

.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}

--></STYLE></HEAD><BODY>
<SYNC start="9209"><P class="ENCC">
    I <b>do</b> <i>not</i> want to go <u>home</u>.<br />
    I don't like it <i><u><b>there</b></u></i>.
</P></SYNC>
<SYNC start="12312"><P class="ENCC">&nbsp;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_css_inline_style():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE><STYLE TYPE="text/css">
<!--
P { margin-left:  1pt;
    margin-right: 1pt;
    margin-bottom: 2pt;
    margin-top: 2pt;
    text-align: center;
    font-size: 10pt;
    font-family: Arial;
    font-weight: normal;
    font-style: normal;
    color: #ffeedd; }

.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}

--></STYLE></HEAD><BODY>
<SYNC start="9209"><P class="ENCC">
    I <span style="font-weight: bold">do</span> <span style="font-style: italic">not</span> want to go <span style="text-decoration: underline">home</span>.<br />
    I don't like it <span style="font-weight:bold;font-style:italic;text-decoration:underline">there</span>.
</P></SYNC>
<SYNC start="12312"><P class="ENCC">&nbsp;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_css_id_style():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE><STYLE TYPE="text/css">
<!--
P { margin-left:  1pt;
    margin-right: 1pt;
    margin-bottom: 2pt;
    margin-top: 2pt;
    text-align: center;
    font-size: 10pt;
    font-family: Arial;
    font-weight: normal;
    font-style: normal;
    color: #ffeedd; }

#StyleItalic { font-style: italic; }
#StyleBold { font-weight: bold; }
#StyleUnderline { text-decoration: underline; }
#StyleItalicBoldUnderline { font-style: italic; font-weight: bold; text-decoration: underline; }

.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}

--></STYLE></HEAD><BODY>
<SYNC start="9209"><P class="ENCC" id="StyleItalic">
    This is in italics.
</P></SYNC>
<SYNC start="12312"><P class="ENCC">&nbsp;</P></SYNC>
<SYNC start="14848"><P class="ENCC" id="StyleUnderline">
    This is underlined.
</P></SYNC>
<SYNC start="17000"><P class="ENCC" id="StyleBold">
    This is bold.
</P></SYNC>
<SYNC start="18752"><P class="ENCC">&nbsp;</P></SYNC>
<SYNC start="20887"><P class="ENCC" id="StyleItalicBoldUnderline">
    This is everything together.
</P></SYNC>
<SYNC start="26760"><P class="ENCC">&nbsp;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_empty():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE><STYLE TYPE="text/css">
<!--
P { margin-left:  1pt;
    margin-right: 1pt;
    margin-bottom: 2pt;
    margin-top: 2pt;
    text-align: center;
    font-size: 10pt;
    font-family: Arial;
    font-weight: normal;
    font-style: normal;
    color: #ffeedd; }

.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}

--></STYLE></HEAD><BODY>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_syntax_error():
    return """
<SAMI>
<Head>
<title>ir2014_111</title>
  <STYLE TYPE="text/css">
    <!--

    P { margin-left:  1pt;
      margin-right: 1pt;
      margin-bottom: 2pt;
      margin-top: 2pt;
      text-align: center;
      font-size: 10pt;
      font-family: Arial;
      font-weight: normal;
      font-style: normal;
      color: #ffffff; }

    #Small {Name:SmallTxt; font-family:Arial;font-weight:normal;font-size:10pt;color:#ffffff;}
    #Big {Name:BigTxt; font-family:Arial;font-weight:bold;font-size:12pt;color:#ffffff;}

    .ENCC {Name:English; lang: en-US; SAMI_Type: CC;}

    -->

  </Style>

</Head>

<BODY>

<Sync Start=0><P Class=ENCC>
<Sync Start=5905><P Class=ENCC>>>> PRESENTATION OF "IDAHO<br>REPORTS" ON IDAHO PUBLIC
<Sync Start=7073><P Class=ENCC>TELEVISION IS MADE POSSIBLE<br>THROUGH THE GENEROUS SUPPORT OF

</Body>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_double_br():
    return """
<SAMI><HEAD><TITLE>NOVA3213</TITLE>
</HEAD><BODY>
<SYNC start="14848"><P class="ENCC">
    MAN:<br/><br/>
    When we think<br/>
    of "E equals m c-squared",
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_partial_margins():
    return """
<SAMI>
<HEAD>
   <STYLE TYPE="Text/css">
   <!--
      P {margin-left: 29pt; margin-right: 29pt; font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
      .SUBTTL {Name: 'Subtitles'; Lang: en-US; SAMIType: CC;}
   -->
   </STYLE>
</HEAD>
<BODY>
   <SYNC START=133>
      <P CLASS=SUBTTL>>> COMING UP NEXT, IT IS<br>APPLAUSE AMERICA.
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_partial_margins_relativized():
    return """<sami>
 <head>
  <style type="text/css">
   <!--
    p {
     background-color: #000;
     color: #ffffff;
     font-family: Tahoma;
     font-size: 24pt;
     font-weight: bold;
     margin-bottom: 0%;
     margin-left: 6.04%;
     margin-right: 6.04%;
     margin-top: 0%;
     text-align: center;
    }

    .subttl {
     lang: en-US;
     margin-bottom: 0%;
     margin-left: 6.04%;
     margin-right: 6.04%;
     margin-top: 0%;
     name: "Subtitles";
     samitype: CC;
    }
   -->
  </style>
 </head>
 <body>
  <sync start="133">
   <p class="subttl" p_style="class:subttl;">
    &gt;&gt; COMING UP NEXT, IT IS<br/>
    APPLAUSE AMERICA.
   </p>
  </sync>
 </body>
</sami>"""


@pytest.fixture(scope="session")
def sample_sami_lang_margin():
    return """
<SAMI>
<HEAD>
   <STYLE TYPE="Text/css">
   <!--
      P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
      .SUBTTL {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
   -->
   </STYLE>
</HEAD>
<BODY>
   <SYNC START=133>
      <P CLASS=SUBTTL>>> COMING UP NEXT, IT IS<br>APPLAUSE AMERICA.
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_span():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC">
            <SPAN Style="font-size:36pt;">we have this vision of Einstein</SPAN>
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_bad_span_align():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC">
            Some say <SPAN Style="text-align:right;">we have this vision of Einstein</SPAN> as an old, wrinkly man
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_bad_div_align():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC">
            Some say <DIV Style="text-align:right;">we have this vision of Einstein</DIV> as an old, wrinkly man
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_p_align():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC" Style="text-align:right;">
            Some say we have this vision of Einstein as an old, wrinkly man
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_p_and_span_align():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC" Style="text-align:right;">
            <SPAN Style="text-align:left;">Some say we have this vision of Einstein as an old, wrinkly man</SPAN>
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_multiple_span_aligns():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC">
            <SPAN Style="text-align:right">Some say </SPAN>
            <SPAN Style="text-align:left;">we have this vision of Einstein </SPAN>
            as an old, wrinkly man
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_no_lang():
    return """
<SAMI>
<Head><STYLE TYPE="text/css"></Style></Head>
<BODY>
<Sync Start=0><P Class=ENCC></p></sync>
<Sync Start=1301><P Class=ENCC>>> FUNDING FOR OVERHEARD</p></sync>
</Body>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_lang():
    return """
<sami>
<head>
<style type="text/css"><!--.en-US {lang: en-US;}--></style>
</head>
<body>
<sync start="1301"><p class="en-US">&gt;&gt; FUNDING FOR OVERHEARD</p></sync>
</body>
</sami>
"""


@pytest.fixture(scope="session")
def sample_sami_with_multi_lang():
    return """
<sami>
<head>
<style type="text/css"><!--.en-US {lang: en-US;} .de-DE {lang: de-DE;}--></style>
</head>
<body>
<sync start="14848">
    <p class="en-US">Butterfly.</p>
    <p class="de-DE">Schmetterling.</p>
</sync>
</body>
</sami>
"""


@pytest.fixture(scope="session")
def sample_sami_with_multiple_p():
    return """
<SAMI>
<HEAD>
    <STYLE TYPE="Text/css">
    <!--
        P {font-size: 24pt; text-align: center; font-family: Tahoma; font-weight: bold; color: #FFFFFF; background-color: #000000;}
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE>
</HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC" Style="text-align:right;">
            1st paragraph.
        </P>
        <P class="ENCC" Style="text-align:left;">
           2nd paragraph.
        </P>
    </SYNC>
    <SYNC start="1337">
        <P class="ENCC" Style="text-align:right;">
            3rd paragraph.
        </P>
    </SYNC>
</BODY>
</SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_empty_cue_output():
    return """
<sami>
 <head>
  <style type="text/css">
   <!--
    .en-US {
     lang: en-US;
    }
   -->
  </style>
 </head>
 <body>
  <sync start="1209">
   <p class="en-US">
    abc
   </p>
  </sync>
 </body>
</sami>
"""


@pytest.fixture(scope="session")
def sample_sami_with_invalid_inline_style():
    return """
<SAMI><HEAD>
    <STYLE TYPE="text/css">
    <!--
        .ENCC {Name: 'Subtitles'; Lang: en-US; SAMIType: CC; margin-top: 20px; margin-right: 20px; margin-bottom: 20px; margin-left: 20px;}
    -->
    </STYLE></HEAD>
<BODY>
    <SYNC start="133">
        <P class="ENCC" Style="text-align:right:font-style:italic">
            Some say we have this vision of Einstein as an old, wrinkly man
        </P>
    </SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_including_hexadecimal_charref():
    return """
<SAMI><HEAD><STYLE TYPE="text/css">
<!--
.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}
--></STYLE></HEAD><BODY>
<SYNC start="101"><P class="ENCC">&#x3E; &#x3E;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_including_decimal_charref():
    return """
<SAMI><HEAD><STYLE TYPE="text/css">
<!--
.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}
--></STYLE></HEAD><BODY>
<SYNC start="101"><P class="ENCC">&#62; &#62;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_including_html5_entityref():
    return """
<SAMI><HEAD><STYLE TYPE="text/css">
<!--
.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}
--></STYLE></HEAD><BODY>
<SYNC start="1301"><P class="ENCC">&starf;_&starf;</P></SYNC>
</BODY></SAMI>
"""


@pytest.fixture(scope="session")
def sample_sami_with_unclosed_tag():
    return """
<SAMI><HEAD><STYLE TYPE="text/css">
<!--
.ENCC {Name: English; lang: en-US; SAMI_Type: CC;}
--></STYLE></HEAD><BODY>
<SYNC start="1101"><P class="ENCC">.</P></SYNC>
</BODY>
"""


@pytest.fixture(scope="session")
def sample_sami_with_inline_lang():
    return """
<SAMI><HEAD></HEAD><BODY>
<SYNC start="1201"><P lang="en-US">Inlined.</P></SYNC>
</BODY></SAMI>
"""


# we do not seem to support nested spans, update this if fixed.
@pytest.fixture(scope="session")
def sample_sami_from_dfxp_with_nested_spans():
    return """<sami>
 <head>
  <style type="text/css">
   <!--
    .s1 {
     font-style: italic;
    }
    .s2 {
     font-weight: bold;
    }
    .s3 {
     text-decoration: underline;
    }
    .en-US {
     lang: en-US;
    }
   -->
  </style>
 </head>
 <body>
  <sync start="3209">
   <p class="en-US">
    That is  <span class="s3" style="classes:['s3'];class:s3;"></span> <span class="s2" style="classes:['s2'];class:s2;"></span> <span class="s1" style="classes:['s1'];class:s1;">nested</span> .
   </p>
  </sync>
 </body>
</sami>"""


@pytest.fixture(scope="session")
def sample_sami_with_separate_multi_lang():
    return """<sami>
 <head>
  <style type="text/css">
   <!--
    .en-UK {
     lang: en-UK;
    }
    .en-US {
     lang: en-US;
    }
   -->
  </style>
 </head>
 <body>
  <sync start="1209">
   <p class="en-UK">
    British text.
   </p>
  </sync>
  <sync start="3209">
   <p class="en-US">
    English text.
   </p>
  </sync>
  <sync start="7209">
   <p class="en-UK">
    OTHER British text.
   </p>
  </sync>
 </body>
</sami>
"""



from pycaption import MicroDVDReader, MicroDVDWriter, SAMIReader

from tests.mixins import MicroDVDTestingMixIn


class TestMicroDVDtoMicroDVD(MicroDVDTestingMixIn):
    def test_microdvd_to_microdvd_conversion(self, sample_microdvd):
        caption_set = MicroDVDReader().read(sample_microdvd)
        results = MicroDVDWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_microdvd_equals(sample_microdvd, results)


class TestSAMItoMicroDVD(MicroDVDTestingMixIn):
    def test_sami_to_micro_dvd_conversion(self, sample_microdvd_2, sample_sami):
        caption_set = SAMIReader().read(sample_sami)
        results = MicroDVDWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_microdvd_equals(sample_microdvd_2, results)



from pycaption import (
    DFXPReader, SAMIReader, SAMIWriter, SRTReader, WebVTTReader,
)

from .mixins import SAMITestingMixIn

# Arbitrary values used to test relativization
VIDEO_WIDTH = 640
VIDEO_HEIGHT = 360


class TestDFXPtoSAMI(SAMITestingMixIn):
    def setup_method(self):
        self.reader = DFXPReader()
        self.writer = SAMIWriter()

    def test_dfxp_to_sami_conversion(self, sample_sami, sample_dfxp):
        caption_set = self.reader.read(sample_dfxp)
        result = self.writer.write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami, result)

    def test_dfxp_to_sami_with_nested_spans(
            self, sample_sami_from_dfxp_with_nested_spans,
            sample_dfxp_with_nested_spans):
        caption_set = self.reader.read(sample_dfxp_with_nested_spans)
        result = self.writer.write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami_from_dfxp_with_nested_spans,
                                        result)

    def test_dfxp_to_sami_with_margins(
            self, sample_dfxp_from_sami_with_margins):
        caption_set = self.reader.read(sample_dfxp_from_sami_with_margins)
        result = SAMIWriter(video_width=VIDEO_WIDTH,
                            video_height=VIDEO_HEIGHT).write(caption_set)
        margins = ["margin-right: 6.04%;",
                   "margin-bottom: 0%;",
                   "margin-top: 0%;",
                   "margin-left: 6.04%;"]

        assert all(margin in result for margin in margins)

    def test_dfxp_empty_cue_to_sami(self, sample_sami_empty_cue_output,
                                    sample_dfxp_empty_cue):
        caption_set = self.reader.read(sample_dfxp_empty_cue)
        result = self.writer.write(caption_set)

        self.assert_sami_captions_equal(sample_sami_empty_cue_output, result)


class TestSRTtoSAMI(SAMITestingMixIn):
    def test_srt_to_sami_conversion(self, sample_sami, sample_srt):
        caption_set = SRTReader().read(sample_srt)
        result = SAMIWriter().write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami, result)


class TestSAMItoSAMI(SAMITestingMixIn):
    def setup_method(self):
        self.reader = SAMIReader()
        self.writer = SAMIWriter()

    def test_sami_to_sami_conversion(self, sample_sami):
        caption_set = self.reader.read(sample_sami)
        result = SAMIWriter(relativize=False,
                            fit_to_screen=False).write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami, result)

    def test_sami_with_multi_lang(self, sample_sami_with_separate_multi_lang):
        caption_set = self.reader.read(sample_sami_with_separate_multi_lang)
        result = self.writer.write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami_with_separate_multi_lang,
                                        result)

    def test_is_relativized(self, sample_sami_partial_margins_relativized,
                            sample_sami_partial_margins):
        # Absolute positioning settings (e.g. px) are converted to percentages
        caption_set = self.reader.read(sample_sami_partial_margins)
        result = SAMIWriter(
            video_width=VIDEO_WIDTH, video_height=VIDEO_HEIGHT
        ).write(caption_set)

        self.assert_sami_captions_equal(sample_sami_partial_margins_relativized,
                                        result)

    def test_missing_language_conversion(self, sample_sami_with_lang,
                                         sample_sami_no_lang):
        caption_set = self.reader.read(sample_sami_no_lang)
        result = self.writer.write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami_with_lang, result)
        assert "lang: und;" in result


class TestWebVTTtoSAMI(SAMITestingMixIn):
    def test_webvtt_to_sami_conversion(self, sample_sami, sample_webvtt):
        caption_set = WebVTTReader().read(sample_webvtt)
        result = SAMIWriter().write(caption_set)

        assert isinstance(result, str)
        self.assert_sami_captions_equal(sample_sami, result)



from copy import deepcopy

import pytest

from pycaption import SAMIReader, CaptionReadNoCaptions, CaptionReadSyntaxError
from pycaption.geometry import HorizontalAlignmentEnum, Size, UnitEnum  # noqa
from tests.mixins import ReaderTestingMixIn


class TestSAMIReader(ReaderTestingMixIn):
    def setup_method(self):
        self.reader = SAMIReader()

    def test_positive_answer_for_detection(self, sample_sami):
        super().assert_positive_answer_for_detection(sample_sami)

    @pytest.mark.parametrize('different_sample', [
        pytest.lazy_fixture('sample_dfxp'),
        pytest.lazy_fixture('sample_microdvd'),
        pytest.lazy_fixture('sample_scc_pop_on'),
        pytest.lazy_fixture('sample_srt'),
        pytest.lazy_fixture('sample_webvtt')
    ])
    def test_negative_answer_for_detection(self, different_sample):
        super().assert_negative_answer_for_detection(different_sample)

    def test_caption_length(self, sample_sami):
        caption_set = self.reader.read(sample_sami)

        assert 7 == len(caption_set.get_captions("en-US"))

    def test_proper_timestamps(self, sample_sami):
        caption_set = self.reader.read(sample_sami)
        paragraph = caption_set.get_captions("en-US")[2]

        assert 17000000 == paragraph.start
        assert 18752000 == paragraph.end

    def test_6digit_color_code_from_6digit_input(self, sample_sami):
        caption_set = self.reader.read(sample_sami)
        p_style = caption_set.get_style("p")

        assert "#ffeedd" == p_style['color']

    def test_6digit_color_code_from_3digit_input(self, sample_sami):
        sample_sami = deepcopy(sample_sami)
        caption_set = self.reader.read(sample_sami.replace("#ffeedd", "#fed"))
        p_style = caption_set.get_style("p")

        assert "#ffeedd" == p_style['color']

    def test_empty_file(self, sample_sami_empty):
        with pytest.raises(CaptionReadNoCaptions):
            self.reader.read(sample_sami_empty)

    def test_invalid_markup_is_properly_handled(self, sample_sami_syntax_error):
        caption_set = self.reader.read(sample_sami_syntax_error)

        assert 2 == len(caption_set.get_captions("en-US"))

    def test_partial_margins(self, sample_sami_partial_margins):
        caption_set = self.reader.read(sample_sami_partial_margins)
        # Ensure that undefined margins are converted to explicitly nil padding
        # (i.e. "0%")

        assert caption_set.layout_info.padding.to_xml_attribute() == \
               '0% 29pt 0% 29pt'

    def test_sami_with_bad_span_align(self, sample_sami_with_bad_span_align):
        caption_set = self.reader.read(sample_sami_with_bad_span_align)
        caption = caption_set.get_captions('en-US')[0]

        assert caption.layout_info.alignment.horizontal == \
               HorizontalAlignmentEnum.RIGHT

    def test_sami_with_bad_div_align(self, sample_sami_with_bad_div_align):
        caption_set = self.reader.read(sample_sami_with_bad_div_align)
        caption = caption_set.get_captions('en-US')[0]

        assert caption.layout_info.alignment.horizontal == \
               HorizontalAlignmentEnum.RIGHT

    def test_sami_with_p_align(self, sample_sami_with_p_align):
        caption_set = self.reader.read(sample_sami_with_p_align)
        caption = caption_set.get_captions('en-US')[0]

        assert caption.layout_info.alignment.horizontal == \
               HorizontalAlignmentEnum.RIGHT

    def test_sami_with_p_and_span_align(self,
                                        sample_sami_with_p_and_span_align):
        """<span> align DOES NOT override <p> align if it is specified inline.
        """
        caption_set = self.reader.read(sample_sami_with_p_and_span_align)
        caption = caption_set.get_captions('en-US')[0]

        assert caption.layout_info.alignment.horizontal == \
               HorizontalAlignmentEnum.RIGHT

    def test_sami_with_invalid_inline_style(
            self, sample_sami_with_invalid_inline_style):
        caption_set = self.reader.read(sample_sami_with_invalid_inline_style)
        caption = caption_set.get_captions("en-US")[0]

        assert caption.layout_info.alignment is None

    def test_sami_including_hexadecimal_charref(
            self, sample_sami_including_hexadecimal_charref):
        caption_set = self.reader.read(
            sample_sami_including_hexadecimal_charref)
        paragraph = caption_set.get_captions("en-US")[0]

        assert '> >' == paragraph.get_text()

    def test_sami_including_decimal_charref(
            self, sample_sami_including_decimal_charref):
        caption_set = self.reader.read(sample_sami_including_decimal_charref)
        paragraph = caption_set.get_captions("en-US")[0]

        assert '> >' == paragraph.get_text()

    def test_sami_including_html5_entityref(
            self, sample_sami_including_html5_entityref):
        caption_set = self.reader.read(sample_sami_including_html5_entityref)
        paragraph = caption_set.get_captions("en-US")[0]

        assert '&starf_&starf' == paragraph.get_text()

    def test_html_file(self):
        with pytest.raises(CaptionReadSyntaxError) as exc_info:
            self.reader.read("<html><head></head><body></body></html>")
        assert exc_info.value.args[0] == 'SAMI File seems to be an HTML file.'

    def test_no_cc_available(self):
        no_cc = 'no closed captioning available'
        with pytest.raises(CaptionReadSyntaxError) as exc_info:
            self.reader.read(f"<SAMI>{no_cc}</SAMI>")
        assert exc_info.value.args[0] == f'SAMI File contains "{no_cc}"'

    def test_sami_with_unclosed_tag(self, sample_sami_with_unclosed_tag):
        caption_set = self.reader.read(sample_sami_with_unclosed_tag)
        paragraph = caption_set.get_captions("en-US")[0]

        assert '.' == paragraph.get_text()

    def test_sami_with_inline_lang(self, sample_sami_with_inline_lang):
        caption_set = self.reader.read(sample_sami_with_inline_lang)
        paragraph = caption_set.get_captions("en")[0]

        assert 'Inlined.' == paragraph.get_text()

    def test_proper_with_timestamps_with_multiple_paragraph(
            self, sample_sami_with_multiple_p):
        captions = self.reader.read(sample_sami_with_multiple_p)
        paragraph_1 = captions.get_captions("en-US")[0]
        paragraph_2 = captions.get_captions("en-US")[1]

        assert paragraph_1.start == paragraph_2.start
        assert paragraph_1.end == paragraph_2.end



#!/usr/bin/env python
import os
from setuptools import setup, find_packages

README_PATH = os.path.join(
    os.path.abspath(os.path.dirname(__file__)),
    'README.rst',
)

dependencies = [
    'beautifulsoup4>=4.8.1,<4.10',
    'lxml>=4.6.3,<4.7',
    'cssutils>=2.0.0,<2.4',
]

setup(
    name='pycaption',
    version='2.0.3',
    description='Closed caption converter',
    long_description=open(README_PATH).read(),
    author='Joe Norton',
    author_email='joey@nortoncrew.com',
    url='https://github.com/pbs/pycaption',
    python_requires='>=3.6,<4.0',
    install_requires=dependencies,
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Multimedia :: Video',
    ],
    test_suite="tests",
)



from bs4 import BeautifulSoup

from pycaption import (
    DFXPReader, SAMIReader, SRTReader, DFXPWriter, WebVTTWriter, MicroDVDWriter,
)
from pycaption.dfxp.extras import LegacyDFXPWriter
from pycaption.dfxp.base import (
    DFXP_DEFAULT_STYLE, DFXP_DEFAULT_STYLE_ID, DFXP_DEFAULT_REGION,
    DFXP_DEFAULT_REGION_ID, _recreate_style, _convert_layout_to_attributes
)

from .mixins import DFXPTestingMixIn, WebVTTTestingMixIn, MicroDVDTestingMixIn

# Arbitrary values used to test relativization
VIDEO_WIDTH = 640
VIDEO_HEIGHT = 360


class TestDFXPtoDFXP(DFXPTestingMixIn):
    def test_dfxp_to_dfxp_conversion(self, sample_dfxp_output, sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        results = DFXPWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_dfxp_equals(sample_dfxp_output, results)

    def test_empty_cue(self, sample_dfxp_empty_cue_output,
                       sample_dfxp_empty_cue):
        caption_set = DFXPReader().read(sample_dfxp_empty_cue)
        results = DFXPWriter().write(caption_set)

        self.assert_dfxp_equals(sample_dfxp_empty_cue_output, results)

    def test_default_styling_tag(self, sample_dfxp_without_region_and_style):
        caption_set = DFXPReader().read(sample_dfxp_without_region_and_style)
        result = DFXPWriter().write(caption_set)

        default_style = _recreate_style(DFXP_DEFAULT_STYLE, None)
        default_style['xml:id'] = DFXP_DEFAULT_STYLE_ID

        soup = BeautifulSoup(result, 'lxml-xml')
        style = soup.find('style', {'xml:id': DFXP_DEFAULT_STYLE_ID})

        assert style
        assert style.attrs == default_style

    def test_default_styling_p_tags(self, sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        result = DFXPWriter().write(caption_set)

        soup = BeautifulSoup(result, 'lxml')

        for p in soup.find_all('p'):
            assert p.attrs.get('style') == 'p'

    def test_default_region_tag(self, sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        result = DFXPWriter().write(caption_set)

        soup = BeautifulSoup(result, 'lxml-xml')
        region = soup.find('region', {'xml:id': DFXP_DEFAULT_REGION_ID})

        default_region = _convert_layout_to_attributes(DFXP_DEFAULT_REGION)
        default_region['xml:id'] = DFXP_DEFAULT_REGION_ID

        assert region
        assert region.attrs['xml:id'] == DFXP_DEFAULT_REGION_ID
        assert region.attrs == default_region

    def test_default_region_p_tags(self, sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        result = DFXPWriter().write(caption_set)

        soup = BeautifulSoup(result, 'lxml')

        for p in soup.find_all('p'):
            assert p.attrs.get('region') == DFXP_DEFAULT_REGION_ID

    def test_correct_region_attributes_are_recreated(
            self, sample_dfxp_multiple_regions_output,
            sample_dfxp_multiple_regions_input):
        caption_set = DFXPReader().read(sample_dfxp_multiple_regions_input)
        result = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(result, sample_dfxp_multiple_regions_output)

    def test_incorrectly_specified_positioning_is_explicitly_accepted(
            self, sample_dfxp_invalid_but_supported_positioning_output,
            sample_dfxp_invalid_but_supported_positioning_input):
        # The arguments used here illustrate how we will try to read
        # and write incorrectly specified positioning information.
        # By incorrect, I mean the specs say that those attributes should be
        # ignored, not that the attributes themselves are outside of the specs
        caption_set = DFXPReader(read_invalid_positioning=True).read(
            sample_dfxp_invalid_but_supported_positioning_input
        )
        result = DFXPWriter(
            relativize=False,
            fit_to_screen=False,
            write_inline_positioning=True).write(caption_set)

        self.assert_dfxp_equals(
            result, sample_dfxp_invalid_but_supported_positioning_output
        )

    def test_dont_create_style_tags_with_no_id(
            self, sample_dfxp_style_tag_with_no_xml_id_output,
            sample_dfxp_style_tag_with_no_xml_id_input):
        # The <style> tags can have no 'xml:id' attribute. Previously, in this
        # case, the style was copied to the output file, with the 'xml:id'
        # property declared, but no value assigned to it. Since such a style
        # can not be referred anyway, and <style> elements, children of
        # <region> tags shouldn't be referred to anyway, we don't include
        # these styles in the output file
        caption_set = DFXPReader().read(
            sample_dfxp_style_tag_with_no_xml_id_input)
        result = DFXPWriter().write(caption_set)

        assert result == sample_dfxp_style_tag_with_no_xml_id_output

    def test_is_relativized(self, sample_dfxp_with_relativized_positioning,
                            sample_dfxp_with_positioning):
        # Absolute positioning settings (e.g. px) are converted to percentages
        caption_set = DFXPReader().read(sample_dfxp_with_positioning)
        result = DFXPWriter(
            video_width=VIDEO_WIDTH, video_height=VIDEO_HEIGHT
        ).write(caption_set)

        assert result == sample_dfxp_with_relativized_positioning

    def test_fit_to_screen(self, sample_dfxp_long_cue_fit_to_screen,
                           sample_dfxp_long_cue):
        # Check if caption width and height are is explicitly set and
        # recalculate it if necessary. This prevents long captions from being
        # cut out of the screen.
        caption_set = DFXPReader().read(sample_dfxp_long_cue)
        result = DFXPWriter().write(caption_set)

        assert result == sample_dfxp_long_cue_fit_to_screen

    def test_proper_xml_entity_escaping(
            self, sample_dfxp_with_escaped_apostrophe):
        caption_set = DFXPReader().read(sample_dfxp_with_escaped_apostrophe)
        cue_text = caption_set.get_captions('en-US')[0].nodes[0].content

        assert cue_text == "<< \"Andy's Caf\xe9 & Restaurant\" this way"
        result = DFXPWriter().write(caption_set)
        assert "&lt;&lt; \"Andy's Café &amp; Restaurant\" this way" in result


class TestDFXPtoWebVTT(WebVTTTestingMixIn):
    def test_dfxp_to_webvtt_conversion(self, sample_webvtt_from_dfxp,
                                       sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_dfxp, results)

    def test_dfxp_with_inline_style_to_webvtt_conversion(
            self, sample_webvtt_from_dfxp_with_style,
            sample_dfxp_with_inline_style):
        caption_set = DFXPReader().read(sample_dfxp_with_inline_style)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_dfxp_with_style, results)

    def test_dfxp_with_defined_style_to_webvtt_conversion(
            self, sample_webvtt_from_dfxp_with_style,
            sample_dfxp_with_defined_style):
        caption_set = DFXPReader().read(sample_dfxp_with_defined_style)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_dfxp_with_style, results)

    def test_dfxp_with_inherited_style_to_webvtt_conversion(
            self, sample_webvtt_from_dfxp_with_style,
            sample_dfxp_with_inherited_style):
        caption_set = DFXPReader().read(sample_dfxp_with_inherited_style)
        results = WebVTTWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(sample_webvtt_from_dfxp_with_style, results)

    def test_dfxp_with_positioning_to_webvtt_conversion(
            self, sample_webvtt_from_dfxp_with_positioning_and_style,
            sample_dfxp_with_positioning):
        caption_set = DFXPReader().read(sample_dfxp_with_positioning)
        results = WebVTTWriter(
            video_width=VIDEO_WIDTH, video_height=VIDEO_HEIGHT
        ).write(caption_set)

        assert isinstance(results, str)
        self.assert_webvtt_equals(
            sample_webvtt_from_dfxp_with_positioning_and_style, results
        )

    def test_dfxp_to_webvtt_adds_explicit_size(
            self, sample_webvtt_output_long_cue, sample_dfxp_long_cue):
        caption_set = DFXPReader().read(sample_dfxp_long_cue)
        results = WebVTTWriter().write(caption_set)

        assert sample_webvtt_output_long_cue == results

    def test_dfxp_to_webvtt_preserves_proper_alignment(
            self, webvtt_from_dfxp_with_conflicting_align,
            dfxp_style_region_align_conflict):
        # This failed at one point when the CaptionSet had node breaks with
        # different positioning. It was fixed both at the DFXPReader AND the
        # WebVTTWriter.
        caption_set = DFXPReader().read(dfxp_style_region_align_conflict)
        results = WebVTTWriter().write(caption_set)

        assert webvtt_from_dfxp_with_conflicting_align == results

    def test_dfxp_empty_cue_to_webvtt(self, sample_webvtt_empty_cue_output,
                                      sample_dfxp_empty_cue):
        caption_set = DFXPReader().read(sample_dfxp_empty_cue)
        results = WebVTTWriter().write(caption_set)

        self.assert_webvtt_equals(sample_webvtt_empty_cue_output, results)


class TestDFXPtoMicroDVD(MicroDVDTestingMixIn):
    def test_dfxp_to_microdvd_conversion(self, sample_microdvd_2, sample_dfxp):
        caption_set = DFXPReader().read(sample_dfxp)
        results = MicroDVDWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_microdvd_equals(sample_microdvd_2, results)

    def test_dfxp_empty_cue_to_microdvd(
            self, sample_microdvd_empty_cue_output, sample_dfxp_empty_cue):
        caption_set = DFXPReader().read(sample_dfxp_empty_cue)
        results = MicroDVDWriter().write(caption_set)

        self.assert_microdvd_equals(sample_microdvd_empty_cue_output, results)


class TestLegacyDFXP:
    def test_legacy_convert(self, sample_dfxp_for_legacy_writer_output,
                            sample_dfxp_for_legacy_writer_input):
        caption_set = DFXPReader(read_invalid_positioning=True).read(
            sample_dfxp_for_legacy_writer_input
        )

        result = LegacyDFXPWriter().write(caption_set)

        assert result == sample_dfxp_for_legacy_writer_output


class TestSAMItoDFXP(DFXPTestingMixIn):
    def test_sami_to_dfxp_conversion(
            self, sample_dfxp_from_sami_with_positioning, sample_sami):
        caption_set = SAMIReader().read(sample_sami)
        results = DFXPWriter(relativize=False,
                             fit_to_screen=False).write(caption_set)

        assert isinstance(results, str)
        self.assert_dfxp_equals(sample_dfxp_from_sami_with_positioning, results)

    def test_sami_to_dfxp_with_margin(self, sample_dfxp_from_sami_with_margins,
                                      sample_sami_partial_margins):
        caption_set = SAMIReader().read(sample_sami_partial_margins)
        results = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(sample_dfxp_from_sami_with_margins, results)

    def test_sami_to_dfxp_with_margin_for_language(
            self, sample_dfxp_from_sami_with_lang_margins,
            sample_sami_lang_margin):
        caption_set = SAMIReader().read(sample_sami_lang_margin)
        results = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(
            sample_dfxp_from_sami_with_lang_margins,
            results
        )

    def test_sami_to_dfxp_with_span(self, sample_dfxp_from_sami_with_span,
                                    sample_sami_with_span):
        caption_set = SAMIReader().read(sample_sami_with_span)
        results = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(sample_dfxp_from_sami_with_span, results)

    def test_sami_to_dfxp_with_bad_span_align(
            self, sample_dfxp_from_sami_with_bad_span_align,
            sample_sami_with_bad_span_align):
        caption_set = SAMIReader().read(sample_sami_with_bad_span_align)
        results = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(
            sample_dfxp_from_sami_with_bad_span_align,
            results
        )

    def test_sami_to_dfxp_ignores_multiple_span_aligns(
            self, sample_dfxp_from_sami_with_bad_span_align,
            sample_sami_with_multiple_span_aligns):
        caption_set = SAMIReader().read(sample_sami_with_multiple_span_aligns)
        results = DFXPWriter(
            relativize=False, fit_to_screen=False).write(caption_set)

        self.assert_dfxp_equals(
            sample_dfxp_from_sami_with_bad_span_align,
            results
        )

    def test_sami_to_dfxp_xml_output(self, sample_sami_syntax_error):
        captions = SAMIReader().read(sample_sami_syntax_error)
        results = DFXPWriter(relativize=False,
                             fit_to_screen=False).write(captions)

        assert isinstance(results, str)
        assert 'xmlns="http://www.w3.org/ns/ttml"' in results
        assert 'xmlns:tts="http://www.w3.org/ns/ttml#styling"' in results


class TestSRTtoDFXP(DFXPTestingMixIn):
    def test_srt_to_dfxp_conversion(self, sample_dfxp, sample_srt):
        caption_set = SRTReader().read(sample_srt)
        results = DFXPWriter().write(caption_set)

        assert isinstance(results, str)
        self.assert_dfxp_equals(
            sample_dfxp, results,
            ignore_styling=True, ignore_spans=True
        )



import pytest


@pytest.fixture(scope="session")
def sample_translated_scc_success():
    return """Scenarist_SCC V1.0

00:00:09:05 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [( ] [cl] [oc] [k ] [ti] [ck] [in] [g ] [)] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:12:08 [Erase Displayed Memory] [Erase Displayed Memory]

00:00:13:18 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 13, column 00, with plain white text.] [row 13, column 00, with plain white text.] [MA] [N:] [row 14, column 00, with plain white text.] [row 14, column 00, with plain white text.] [Wh] [en] [ w] [e ] [th] [in] [k] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [of] [ "] [E ] [eq] [ua] [ls] [ m] [ c] [-s] [qu] [ar] [ed] [",] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:16:03 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [we] [ h] [av] [e ] [th] [is] [ v] [is] [io] [n ] [of] [ E] [in] [st] [ei] [n] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:17:20 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 14, column 00, with plain white text.] [row 14, column 00, with plain white text.] [as] [ a] [n ] [ol] [d,] [ w] [ri] [nk] [ly] [ m] [an] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [wi] [th] [ w] [hi] [te] [ h] [ai] [r.] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:19:13 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 13, column 00, with plain white text.] [row 13, column 00, with plain white text.] [MA] [N ] [2:] [row 14, column 00, with plain white text.] [row 14, column 00, with plain white text.] [E ] [eq] [ua] [ls] [ m] [ c] [-s] [qu] [ar] [ed] [ i] [s] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [no] [t ] [ab] [ou] [t ] [an] [ o] [ld] [ E] [in] [st] [ei] [n.] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:25:16 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 13, column 00, with plain white text.] [row 13, column 00, with plain white text.] [MA] [N ] [2:] [row 14, column 00, with plain white text.] [row 14, column 00, with plain white text.] [It] ['s] [ a] [ll] [ a] [bo] [ut] [ a] [n ] [et] [er] [na] [l] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [Ei] [ns] [te] [in] [.] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:31:15 [Erase Non-displayed Memory] [Erase Non-displayed Memory] [Resume Caption Loading] [Resume Caption Loading] [row 15, column 00, with plain white text.] [row 15, column 00, with plain white text.] [<L] [AU] [GH] [IN] [G ] [& ] [WH] [OO] [PS] [!>] [Erase Displayed Memory] [Erase Displayed Memory] [End Of Caption] [End Of Caption]

00:00:36:04 [Erase Displayed Memory] [Erase Displayed Memory]

"""


@pytest.fixture(scope="session")
def sample_translated_scc_no_brackets():
    return """Scenarist_SCC V1.0

00:00:09:05 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 15, column 00, with plain white text. row 15, column 00, with plain white text. (  cl oc k  ti ck in g  ) Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:12:08 Erase Displayed Memory Erase Displayed Memory

00:00:13:18 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 13, column 00, with plain white text. row 13, column 00, with plain white text. MA N: row 14, column 00, with plain white text. row 14, column 00, with plain white text. Wh en  w e  th in k row 15, column 00, with plain white text. row 15, column 00, with plain white text. of  " E  eq ua ls  m  c -s qu ar ed ", Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:16:03 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 15, column 00, with plain white text. row 15, column 00, with plain white text. we  h av e  th is  v is io n  of  E in st ei n Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:17:20 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 14, column 00, with plain white text. row 14, column 00, with plain white text. as  a n  ol d,  w ri nk ly  m an row 15, column 00, with plain white text. row 15, column 00, with plain white text. wi th  w hi te  h ai r. Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:19:13 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 13, column 00, with plain white text. row 13, column 00, with plain white text. MA N  2: row 14, column 00, with plain white text. row 14, column 00, with plain white text. E  eq ua ls  m  c -s qu ar ed  i s row 15, column 00, with plain white text. row 15, column 00, with plain white text. no t  ab ou t  an  o ld  E in st ei n. Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:25:16 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 13, column 00, with plain white text. row 13, column 00, with plain white text. MA N  2: row 14, column 00, with plain white text. row 14, column 00, with plain white text. It 's  a ll  a bo ut  a n  et er na l row 15, column 00, with plain white text. row 15, column 00, with plain white text. Ei ns te in . Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:31:15 Erase Non-displayed Memory Erase Non-displayed Memory Resume Caption Loading Resume Caption Loading row 15, column 00, with plain white text. row 15, column 00, with plain white text. <L AU GH IN G  &  WH OO PS !> Erase Displayed Memory Erase Displayed Memory End Of Caption End Of Caption

00:00:36:04 Erase Displayed Memory Erase Displayed Memory

"""


@pytest.fixture(scope="session")
def sample_translated_scc_commands_not_found():
    return """Scenarist_SCC V1.0

00:04:36;06 942x 942x 942x 942x [row 01, column 12, with plain white text.] [MA] 13b0 525x c8cx ba8x
"""


@pytest.fixture(scope="session")
def sample_translated_scc_custom_brackets():
    return """Scenarist_SCC V1.0

00:00:09:05 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {( } {cl} {oc} {k } {ti} {ck} {in} {g } {)} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:12:08 {Erase Displayed Memory} {Erase Displayed Memory}

00:00:13:18 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 13, column 00, with plain white text.} {row 13, column 00, with plain white text.} {MA} {N:} {row 14, column 00, with plain white text.} {row 14, column 00, with plain white text.} {Wh} {en} { w} {e } {th} {in} {k} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {of} { "} {E } {eq} {ua} {ls} { m} { c} {-s} {qu} {ar} {ed} {",} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:16:03 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {we} { h} {av} {e } {th} {is} { v} {is} {io} {n } {of} { E} {in} {st} {ei} {n} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:17:20 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 14, column 00, with plain white text.} {row 14, column 00, with plain white text.} {as} { a} {n } {ol} {d,} { w} {ri} {nk} {ly} { m} {an} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {wi} {th} { w} {hi} {te} { h} {ai} {r.} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:19:13 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 13, column 00, with plain white text.} {row 13, column 00, with plain white text.} {MA} {N } {2:} {row 14, column 00, with plain white text.} {row 14, column 00, with plain white text.} {E } {eq} {ua} {ls} { m} { c} {-s} {qu} {ar} {ed} { i} {s} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {no} {t } {ab} {ou} {t } {an} { o} {ld} { E} {in} {st} {ei} {n.} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:25:16 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 13, column 00, with plain white text.} {row 13, column 00, with plain white text.} {MA} {N } {2:} {row 14, column 00, with plain white text.} {row 14, column 00, with plain white text.} {It} {'s} { a} {ll} { a} {bo} {ut} { a} {n } {et} {er} {na} {l} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {Ei} {ns} {te} {in} {.} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:31:15 {Erase Non-displayed Memory} {Erase Non-displayed Memory} {Resume Caption Loading} {Resume Caption Loading} {row 15, column 00, with plain white text.} {row 15, column 00, with plain white text.} {<L} {AU} {GH} {IN} {G } {& } {WH} {OO} {PS} {!>} {Erase Displayed Memory} {Erase Displayed Memory} {End Of Caption} {End Of Caption}

00:00:36:04 {Erase Displayed Memory} {Erase Displayed Memory}

"""